unit fOrdersRenew;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  fAutoSz, StdCtrls, ORFn, ComCtrls, uConst, rODMeds;

type
  TfrmRenewOrders = class(TfrmAutoSz)
    lstOrders: TListBox;
    cmdOK: TButton;
    cmdCancel: TButton;
    cmdChange: TButton;
    hdrOrders: THeaderControl;
    procedure FormCreate(Sender: TObject);
    procedure cmdOKClick(Sender: TObject);
    procedure cmdCancelClick(Sender: TObject);
    procedure lstOrdersMeasureItem(Control: TWinControl; Index: Integer;
      var Height: Integer);
    procedure lstOrdersDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure lstOrdersClick(Sender: TObject);
    procedure cmdChangeClick(Sender: TObject);
  private
    OKPressed: Boolean;
    OrderList: TList;
  end;

function ExecuteRenewOrders(var SelectedList: TList): Boolean;

implementation

{$R *.DFM}

uses rOrders, fDateRange, fRenewOutMed, uCore;

const
  TEXT_COLUMN = 0;
  DATE_COLUMN = 1;
  WORD_WRAPPED = True;

  TC_START_STOP = 'Change Start/Stop Dates';
  TX_START_STOP = 'Enter the start and stop times for this order.  Stop time is optional.';
  TX_LBL_START  = 'Start Date/Time';
  TX_LBL_STOP   = 'Stop Date/Time';
  TX_NO_DEA     = 'Provider must have a DEA# or VA# to review this order';
  TC_NO_DEA     = 'DEA# Required';

function PickupText(const x: string): string;
begin
  case CharAt(x, 1) of
  'C': Result := ' (administered in Clinic)';
  'M': Result := ' (deliver by Mail)';
  'W': Result := ' (pick up at Window)';
  else Result := '';
  end;
end;

function ExecuteRenewOrders(var SelectedList: TList): Boolean;
var
  frmRenewOrders: TfrmRenewOrders;
  RenewFields: TOrderRenewFields;
  AnOrder, TheOrder: TOrder;
  OriginalID: string;
  OrderableItemIen: integer;
  PassDeaList: TList;
  IsInpt: boolean;
  i,j: Integer;
  PkgInfo:string;

  function OrderForInpatient: Boolean;
  begin
    Result := Patient.Inpatient;
  end;
begin
  Result := False;
  if SelectedList.Count = 0 then Exit;
  PassDeaList := TList.Create;
  frmRenewOrders := TfrmRenewOrders.Create(Application);
  try
    ResizeFormToFont(TForm(frmRenewOrders));
    IsInpt := OrderForInpatient;
    with SelectedList do for j := 0 to Count - 1 do
    begin
      TheOrder := TOrder(Items[j]);
      PkgInfo := GetPackageByOrderID(TheOrder.ID);
      if Pos('PS',PkgInfo)=1 then
      begin
        OrderableItemIen := GetOrderableIen(TheOrder.ID);
        if DEACheckFailed(OrderableItemIen, IsInPt) then
        begin
          InfoBox(TX_NO_DEA + #13 + TheOrder.Text, TC_NO_DEA, MB_OK);
          UnlockOrder(TheOrder.ID);
        end else PassDeaList.Add(SelectedList.Items[j]);
      end else
        PassDeaList.Add(SelectedList.Items[j]);
    end;
    SelectedList.Clear;
    SelectedList := PassDeaList;
    frmRenewOrders.OrderList := SelectedList;
    with SelectedList do for i := 0 to Count - 1 do
    begin
      AnOrder := TOrder(Items[i]);
      RenewFields := TOrderRenewFields.Create;
      LoadRenewFields(RenewFields, AnOrder.ID);
      RenewFields.NewText := AnOrder.Text + PickupText(RenewFields.Pickup);
      AnOrder.LinkObject := RenewFields;
      frmRenewOrders.lstOrders.Items.AddObject(AnOrder.ID, AnOrder);
    end;
    if SelectedList.Count < 1 then
      frmRenewOrders.Close
    else
      frmRenewOrders.ShowModal;
    if frmRenewOrders.OKPressed then
    begin
      with SelectedList do for i := 0 to Count - 1 do
      begin
        AnOrder := TOrder(Items[i]);
        OriginalID := AnOrder.ID;
        RenewFields := TOrderRenewFields(AnOrder.LinkObject);
        RenewOrder(AnOrder, RenewFields);
        AnOrder.ActionOn := OriginalID + '=RN';
        SendMessage(Application.MainForm.Handle, UM_NEWORDER, ORDER_ACT, Integer(AnOrder));
      end;
      Result := True;
    end
    else with SelectedList do for i := 0 to Count - 1 do UnlockOrder(TOrder(Items[i]).ID);
  finally
    // free all the TOrderRenewFields that were created
    with SelectedList do for i := 0 to Count - 1 do
    begin
      AnOrder := TOrder(Items[i]);
      RenewFields := TOrderRenewFields(AnOrder.LinkObject);
      RenewFields.Free;
      AnOrder.LinkObject := nil;
    end;
    frmRenewOrders.Release;
  end;
end;

procedure TfrmRenewOrders.FormCreate(Sender: TObject);
begin
  inherited;
  lstOrders.Color := ReadOnlyColor;
  OKPressed := False;
end;

procedure TfrmRenewOrders.lstOrdersMeasureItem(Control: TWinControl;
  Index: Integer; var Height: Integer);
var
  x: string;
  DateHeight, TextHeight: Integer;
  AnOrder: TOrder;
  RenewFields: TOrderRenewFields;
begin
  inherited;
  AnOrder := TOrder(OrderList.Items[Index]);
  if AnOrder <> nil then
  begin
    RenewFields := TOrderRenewFields(AnOrder.LinkObject);
    with RenewFields do x := 'Start: ' + StartTime + CRLF + 'Stop: ' + StopTime;
    DateHeight := ListGridRowHeight(lstOrders, hdrOrders, Index, DATE_COLUMN, x);
    TextHeight := ListGridRowHeight(lstOrders, hdrOrders, Index, TEXT_COLUMN, RenewFields.NewText);
    Height := HigherOf(TextHeight, DateHeight);
  end
end;

procedure TfrmRenewOrders.lstOrdersDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
var
  x: string;
  AnOrder: TOrder;
  RenewFields: TOrderRenewFields;
begin
  inherited;
  AnOrder := TOrder(lstOrders.Items.Objects[Index]);
  if AnOrder <> nil then with AnOrder do
  begin
    RenewFields := TOrderRenewFields(AnOrder.LinkObject);
    if RenewFields.BaseType = OD_TEXTONLY
      then with RenewFields do x := 'Start: ' + StartTime + CRLF + 'Stop: ' + StopTime
      else x := '';
    ListGridDrawLines(lstOrders, hdrOrders, Index, State);
    ListGridDrawCell(lstOrders, hdrOrders, Index, TEXT_COLUMN, RenewFields.NewText, WORD_WRAPPED);
    ListGridDrawCell(lstOrders, hdrOrders, Index, DATE_COLUMN, x, WORD_WRAPPED);
  end;
end;

procedure TfrmRenewOrders.lstOrdersClick(Sender: TObject);
var
  RenewFields: TOrderRenewFields;
begin
  inherited;
  with lstOrders do
  begin
    if ItemIndex < 0 then Exit;
    RenewFields := TOrderRenewFields(TOrder(Items.Objects[ItemIndex]).LinkObject);
    case RenewFields.BaseType of
    OD_MEDOUTPT: cmdChange.Caption := 'Change Refills/Pick Up...';
    OD_TEXTONLY: cmdChange.Caption := 'Change Start/Stop...';
    else         cmdChange.Caption := 'Change...';
    end;
    with RenewFields do if (BaseType = OD_MEDOUTPT) or (BaseType = OD_TEXTONLY)
      then cmdChange.Enabled := True
      else cmdChange.Enabled := False;
  end;
end;

procedure TfrmRenewOrders.cmdChangeClick(Sender: TObject);
var
  StartPos: Integer;
  x, NewComment, OldComment, OldRefills, OldPickup: string;
  AnOrder: TOrder;
  RenewFields: TOrderRenewFields;
begin
  inherited;
  with lstOrders do
  begin
    if ItemIndex < 0 then Exit;
    AnOrder := TOrder(Items.Objects[ItemIndex]);
    RenewFields := TOrderRenewFields(AnOrder.LinkObject);
    case RenewFields.BaseType of
    OD_MEDOUTPT: with RenewFields do begin
                   OldRefills := IntToStr(Refills) + ' refills';
                   { reverse string to make sure getting last matching comment }
                   OldComment := ReverseStr(Comments);
                   OldPickup  := PickupText(Pickup);
                   ExecuteRenewOutMed(Refills, Comments, Pickup, AnOrder);
                   NewComment := ReverseStr(Comments);
                   x := ReverseStr(NewText);
                   StartPos := Pos(OldComment, x);
                   if StartPos > 0
                     then x := Copy(x, 1, StartPos - 1) + NewComment +
                               Copy(x, StartPos + Length(OldComment), Length(x))
                     else x := NewComment + x;
                   NewText := ReverseStr(x);
                   x := NewText;
                   StartPos := Pos(OldRefills, x);
                   if StartPos > 0
                     then x := Copy(x, 1, StartPos - 1) + IntToStr(Refills) + ' refills' +
                               Copy(x, StartPos + Length(OldRefills), Length(x))
                     else x := x + ' ' + IntToStr(Refills) + ' refills';
                   StartPos := Pos(OldPickup, x);
                   if StartPos > 0
                     then x := Copy(x, 1, StartPos - 1) + PickupText(Pickup) +
                               Copy(x, StartPos + Length(OldPickup), Length(x))
                     else x := x + PickupText(Pickup);
                   NewText := x;
                 end;
    OD_TEXTONLY: with RenewFields do ExecuteDateRange(StartTime, StopTime, DT_FUTURE+DT_TIMEOPT,
                   TC_START_STOP, TX_START_STOP, TX_LBL_START, TX_LBL_STOP);
    end;
  end;
  lstOrders.Invalidate;
end;

procedure TfrmRenewOrders.cmdOKClick(Sender: TObject);
begin
  inherited;
  OKPressed := True;
  Close;
end;

procedure TfrmRenewOrders.cmdCancelClick(Sender: TObject);
begin
  inherited;
  Close;
end;

end.
