unit fOrdersSign;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  fAutoSz, StdCtrls, ORFn, ORCtrls, AppEvnts, mCoPayDesc, XUDIGSIGSC_TLB,
  ComCtrls;

type
  TfrmSignOrders = class(TForm)
    lstOrders: TListBox;
    Label1: TLabel;
    cmdOK: TButton;
    cmdCancel: TButton;
    lblESCode: TLabel;
    txtESCode: TEdit;
    fraCoPay: TfraCoPayDesc;
    procedure FormCreate(Sender: TObject);
    procedure cmdOKClick(Sender: TObject);
    procedure cmdCancelClick(Sender: TObject);
    procedure lstOrdersMeasureItem(Control: TWinControl; Index: Integer;
      var Height: Integer);
    procedure lstOrdersMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure lstOrdersDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
  private
    OKPressed: Boolean;
    ESCode: string;
    FLastHintItem: integer;
    FOldHintPause: integer;
    FOldHintHidePause: integer;
  end;

function ExecuteSignOrders(SelectedList: TList): Boolean;

var
  crypto: IXuDigSigS;

implementation

{$R *.DFM}

uses Hash, rCore, rOrders, uConst, fOrdersPrint, uCore, uOrders, uSignItems;

const
  TX_SAVERR1 = 'The error, ';
  TX_SAVERR2 = ', occurred while trying to save:' + CRLF + CRLF;
  TC_SAVERR  = 'Error Saving Order';

function ExecuteSignOrders(SelectedList: TList): Boolean;
var
  frmSignOrders: TfrmSignOrders;
  i, idx: Integer;
  SignList: TStringList;
  Obj: TOrder;
  DigSigErr, DigStoreErr: Boolean;
  SigData, SigUser, SigDrugSch, SigDEA: string;
  cSignature, cHashData, cCrlUrl: string;
  cProvDUZ: Int64;

  OrderText: string;

  function FindOrderText(const AnID: string): string;
  var
    i: Integer;
  begin
    Result := '';
    with SelectedList do for i := 0 to Count - 1 do
      with TOrder(Items[i]) do if ID = AnID then
      begin
        Result := Text;
        Break;
      end;
  end;

  function SignNotRequired: Boolean;
  var
    i: Integer;
  begin
    Result := True;
    with SelectedList do for i := 0 to Pred(Count) do
    begin
      with TOrder(Items[i]) do if Signature <> OSS_NOT_REQUIRE then Result := False;
    end;
  end;

  function DigitalSign: Boolean;
  var
    i: Integer;
  begin
    Result := False;
    if GetPKIUse <> true then exit;
    with SelectedList do for i := 0 to Pred(Count) do
    begin
      with TOrder(Items[i]) do if DigSigReq = '1' then Result := True;
    end;
  end;

begin
  Result := False;
  DigSigErr := True;
  DigStoreErr := False;
  if SelectedList.Count = 0 then Exit;
  frmSignOrders := TfrmSignOrders.Create(Application);
  try
//    ResizeFormToFont(TForm(frmSignOrders));
    SigItems.ResetOrders;
    with SelectedList do for i := 0 to Count - 1 do
    begin
      obj := TOrder(Items[i]);
      idx := frmSignOrders.lstOrders.Items.Add(Obj.Text);
      SigItems.Add(CH_ORD, obj.ID, idx);
    end;
    if SigItems.UpdateListBox(frmSignOrders.lstOrders) then
      frmSignOrders.fraCoPay.Visible := TRUE
    else
      frmSignOrders.height := frmSignOrders.height - frmSignOrders.fraCoPay.Height - 9;

    if GetPKIUse and DigitalSign then //PKI setup for crypto card read
    begin
      try  //PKI object creation
        crypto := CoXuDigSigS.Create;
        crypto.GetCSP;
        StatusText(crypto.Reason);
        DigSigErr := False;
      except
        on  E: Exception do
          begin
            DigSigErr := True;
            ShowMessage('Error while trying to Digitally Sign: '+ piece(E.Message,'^',2));
          end;
      end;
        end;

    if SignNotRequired then
    begin
      frmSignOrders.lblESCode.Visible := False;
      frmSignOrders.txtESCode.Visible := False;
    end;
    frmSignOrders.ShowModal;
    if frmSignOrders.OKPressed then
    begin
      Result := True;
      SignList := TStringList.Create;
      try
        with SelectedList do for i := 0 to Count - 1 do with TOrder(Items[i]) do
          begin
            if (DigSigErr = False) and (TOrder(Items[i]).DigSigReq = '1') then
              begin
                StatusText('Retrieving DIGITAL SIGNATURE');
                SigData := TOrder(Items[i]).XMLText;
                SigUser := TOrder(Items[i]).ProviderName;
                SigDrugSch := TOrder(Items[i]).DigSigReq;
                cProvDUZ := TOrder(Items[i]).Provider;
                if SigDrugSch = '1' then SigDrugSch := '2';
                SigDEA := TOrder(Items[i]).ProviderDEA;
                if (length(SigData)<1) or (length(SigUser)<1) or (length(SigDEA)<1) or (cProvDUZ<1) then
                  begin
                    SignList.Add(ID + U + SS_ESIGNED + U + RS_RELEASE + U + NO_PROVIDER);
                    continue;
                  end;
                try
                  crypto.Reset;
                  crypto.DEAsig := true;
                  crypto.UsrName := SigUser;
                  crypto.DrugSch := SigDrugSch;
                  crypto.UsrNumber := SigDEA;
                  crypto.DataBuffer := SigData;
                  if crypto.Signdata = true then
                    begin
                      cSignature := crypto.Signature;
                      cHashData := crypto.HashValue;
                      cCrlUrl := crypto.CrlUrl;
                    end
                  else
                    begin
                      ShowMessage('Digital Signature failed with reason: '+ piece(Crypto.Reason, '^', 2));
                      DigStoreErr := true;
                    end;
                except
                  on  E: Exception do
                    begin
                      ShowMessage('Crypto raised an error: '+ E.Message);
                      DigStoreErr := true;
                    end;
                end;  //except
                if DigStoreErr = true then
                  SignList.Add(ID + U + SS_ESIGNED + U + RS_RELEASE + U + NO_PROVIDER)
                else
                  begin
                    StoreDigitalSig(ID, cHashData, cProvDUZ, cSignature, cCrlUrl);
                    SignList.Add(ID + U + SS_DIGSIG + U + RS_RELEASE + U + NO_PROVIDER);
                  end;
              end
            else
              SignList.Add(ID + U + SS_ESIGNED + U + RS_RELEASE + U + NO_PROVIDER);
          end;
        StatusText('Sending Orders to Service(s)...');
        if SignList.Count > 0 then
        begin
          SigItems.SaveSettings; // Save CoPay FIRST!
          SendOrders(SignList, frmSignOrders.ESCode);
        end;
        with SignList do if Count > 0 then for i := 0 to Count - 1 do
          begin
            if Pos('E', Piece(SignList[i], U, 2)) > 0 then
              begin
                OrderText := FindOrderText(Piece(SignList[i], U, 1));
                InfoBox(TX_SAVERR1 + Piece(SignList[i], U, 4) + TX_SAVERR2 + OrderText,
                        TC_SAVERR, MB_OK);
              end;
            if Pos('R', Piece(SignList[i], U, 2)) > 0 then
              NotifyOtherApps(NAE_ORDER, 'RL' + U + Piece(SignList[i], U, 1));
          end;
        StatusText('');
        PrintOrdersOnSignRelease(SignList, NO_PROVIDER);
//        SetupOrdersPrint(SignList, DeviceInfo, NO_PROVIDER, False, PrintIt);  //*KCM*
//        if PrintIt then PrintOrdersOnReview(SignList, DeviceInfo);            //*KCM*
        // This is sent after returning to the orders tab, so not done here. -
        // SendMessage(Application.MainForm.Handle, UM_NEWORDER, ORDER_SIGN, 0);
      finally
        SignList.Free;
      end;
    end; {if frmSignOrders.OKPressed}
  finally
    frmSignOrders.Free;
    with SelectedList do for i := 0 to Count - 1 do UnlockOrder(TOrder(Items[i]).ID);
  end;
end;

procedure TfrmSignOrders.FormCreate(Sender: TObject);
begin
  inherited;
  FLastHintItem := -1;
  OKPressed := False;
  FOldHintPause := Application.HintPause;
  Application.HintPause := 250;
  FOldHintHidePause := Application.HintHidePause;
  Application.HintHidePause := 30000;

end;

procedure TfrmSignOrders.cmdOKClick(Sender: TObject);
const
  TX_NO_CODE  = 'An electronic signature code must be entered to sign orders.';
  TC_NO_CODE  = 'Electronic Signature Code Required';
  TX_BAD_CODE = 'The electronic signature code entered is not valid.';
  TC_BAD_CODE = 'Invalid Electronic Signature Code';
begin
  inherited;
  if txtESCode.Visible and (Length(txtESCode.Text) = 0) then
  begin
    InfoBox(TX_NO_CODE, TC_NO_CODE, MB_OK);
    Exit;
  end;
  if txtESCode.Visible and not ValidESCode(txtESCode.Text) then
  begin
    InfoBox(TX_BAD_CODE, TC_BAD_CODE, MB_OK);
    txtESCode.SetFocus;
    txtESCode.SelectAll;
    Exit;
  end;
  if not SigItems.OK2SaveSettings then
  begin
    InfoBox(TX_CoPay_Error, TC_CoPay_Error, MB_OK);
    Exit;
  end;
  if txtESCode.Visible then ESCode := Encrypt(txtESCode.Text) else ESCode := '';
  OKPressed := True;
  Close;
end;

procedure TfrmSignOrders.cmdCancelClick(Sender: TObject);
begin
  inherited;
  Close;
end;


procedure TfrmSignOrders.lstOrdersMeasureItem(Control: TWinControl;
  Index: Integer; var Height: Integer);
begin
  inherited;
  Height := SigItemHeight;
end;

procedure TfrmSignOrders.lstOrdersMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  Itm: integer;

begin
  inherited;
  Itm := lstOrders.ItemAtPos(Point(X, Y), TRUE);
  if (Itm >= 0) then
  begin
    if (Itm <> FLastHintItem) then
    begin
      Application.CancelHint;
      lstOrders.Hint := TrimRight(lstOrders.Items[Itm]);
      FLastHintItem := Itm;
      Application.ActivateHint(Point(X, Y));
    end;
  end
  else
  begin
    lstOrders.Hint := '';
    FLastHintItem := -1;
    Application.CancelHint;
  end;
end;

procedure TfrmSignOrders.FormDestroy(Sender: TObject);
begin
  inherited;
  Application.HintPause := FOldHintPause;
  Application.HintHidePause := FOldHintHidePause;
  Crypto := nil;  //PKI object destroy
end;

procedure TfrmSignOrders.lstOrdersDrawItem(Control: TWinControl;
  Index: Integer; Rect: TRect; State: TOwnerDrawState);
begin
  inherited;
  with lstOrders do
  begin
    Canvas.FillRect(Rect);
    if Index < Items.Count then Canvas.TextOut(Rect.Left + 2, Rect.Top, FilteredString(Items[Index]));
  end;
end;

end.
