unit rODAllergy;

{$O-}

interface

uses SysUtils, Classes, ORNet, ORFn, rCore, uCore, TRPCB, dialogs ;

type
  TAllergyRec = record
    Changed:             boolean;
    IEN:                 Integer;
    CausativeAgent:      string;
    NatureOfReaction:    string;
    SignsSymptoms:       TStringList;
    Originator:          int64;
    OriginatorName:      string;
    Originated:          TFMDateTime;
    SignOff:             boolean;
    Comments:            TStringList;
    IDBandMarked:        boolean;
    ChartMarked:         boolean;
    Verifier:            int64;
    VerifierName:        string;
    Verified:            boolean;
    VerifiedDateTime:    TFMDateTime;
    EnteredInError:      boolean;
    Observed_Historical: string;
    Observations:        TStringList;
    ReactionDate:        TFMDateTime;
    Severity:            string;
    NoKnownAllergies:    Boolean;
    NewComments:         TStringList;
  end;

function SearchForAllergies(StringToMatch: string): TStrings;
function SubsetofSymptoms(const StartFrom: string; Direction: Integer): TStrings;
function ODForAllergies: TStrings;
function GetCWADInfo(const DFN: string): string;
function SaveAllergy(EditRec: TAllergyRec): string;
function LoadAllergyForEdit(AllergyIEN: integer): TAllergyRec;

implementation

function ODForAllergies: TStrings;
begin
  CallV('ORWDAL32 DEF',[nil]);
  Result := RPCBrokerV.Results;
end;

function SearchForAllergies(StringToMatch: string): TStrings;
begin
  CallV('ORWDAL32 ALLERGY MATCH',[StringToMatch]);
  Result := RPCBrokerV.Results;
end;

function SubsetofSymptoms(const StartFrom: string; Direction: Integer): TStrings;
begin
  Callv('ORWDAL32 SYMPTOMS',[StartFrom, Direction]);
  Result := RPCBrokerV.Results;
end;

function GetCWADInfo(const DFN: string): string;
begin
  Result := sCallV('ORWPT CWAD',[DFN]);
end;

function LoadAllergyForEdit(AllergyIEN: integer): TAllergyRec;
var
  Dest: TStringList;
  EditRec: TAllergyRec;
begin
  Dest := TStringList.Create;
  try
    tCallV(Dest, 'ORWDAL32 LOAD FOR EDIT', [AllergyIEN]) ;
    with EditRec do
      begin
        Changed             := False;
        IEN                 := AllergyIEN;
        CausativeAgent      := ExtractDefault(Dest, 'CAUSATIVE AGENT');
        NatureOfReaction    := ExtractDefault(Dest, 'NATURE OF REACTION');
        SignsSymptoms       := TStringList.Create;
        ExtractItems(SignsSymptoms, Dest, 'SIGN/SYMPTOMS');
        MixedCaseByPiece(SignsSymptoms, U, 4);
        Originator          := StrToInt64Def(Piece(ExtractDefault(Dest, 'ORIGINATOR'), U, 1), 0);
        OriginatorName      := Piece(ExtractDefault(Dest, 'ORIGINATOR'), U, 2);
        Originated          := StrToFMDateTime(ExtractDefault(Dest, 'ORIGINATED'));
        SignOff             := ExtractDefault(Dest, 'SIGN OFF') = 'YES';
        Comments            := TStringList.Create;
        ExtractText(Comments, Dest, 'COMMENTS');
        IDBandMarked        := ExtractDefault(Dest, 'ID BAND MARKED') = 'YES';
        ChartMarked         := ExtractDefault(Dest, 'CHART MARKED') = 'YES';
        Verifier            := StrToInt64Def(Piece(ExtractDefault(Dest, 'VERIFIER'), U, 1), 0);
        VerifierName        := Piece(ExtractDefault(Dest, 'VERIFIER'), U, 2);
        Verified            := ExtractDefault(Dest, 'VERIFIED') = 'YES';
        if Verified then
          VerifiedDateTime  := StrToFMDateTime(Piece(ExtractDefault(Dest, 'VERIFIED'), U, 2));
        EnteredInError      := ExtractDefault(Dest, 'ENTERED IN ERROR') = 'YES';
        Observed_Historical := ExtractDefault(Dest, 'OBS/HIST');
        Observations        := TStringList.Create;
        ExtractText(Observations, Dest, 'OBSERVATIONS');
        //ReactionDate        := StrToFMDateTime(Piece(ExtractDefault(Dest, 'REACTDT'), U, 3));
        //Severity            := Piece(ExtractDefault(Dest, 'SEVERITY'), U, 3);
        NoKnownAllergies    := (StrToIntDef(Piece(ExtractDefault(Dest, 'NKA'), U, 3), 0) > 0);
        NewComments         := TStringList.Create;
      end;
    Result := EditRec;
  finally
    Dest.Free;
  end;
end;

function SaveAllergy(EditRec: TAllergyRec): string;
var
  i: integer;
begin
  with RPCBrokerV, EditRec do
    begin
      ClearParameters := True;
      RemoteProcedure := 'ORWDAL32 SAVE ALLERGY';
      Param[0].PType := literal;
      Param[0].Value := IntToStr(IEN);
      Param[1].PType := literal;
      Param[1].Value := Patient.DFN;
      Param[2].PType := list;
      with Param[2] do
        begin
          if NoKnownAllergies then
            Mult['1']  := 'GMRANKA^'  + BOOLCHAR[NoKnownAllergies];
          if CausativeAgent <> '' then
            Mult['2']  := 'GMRAGNT^'  + CausativeAgent;
          if NatureOfReaction <> '' then
            Mult['3']  := 'GMRANATR^' + NatureOfReaction ;
          if Originator > 0 then
            Mult['4']  := 'GMRAORIG^'  + IntToStr(Originator);
          if Originated > 0 then
            Mult['5']  := 'GMRAORDT^'  + FloatToStr(Originated);
          with SignsSymptoms do if Count > 0 then
            begin
              Mult['6'] := 'GMRASYMP^' + IntToStr(Count);
              for i := 0 to Count - 1 do
                Mult['6,' + IntToStr(i+1)] := Piece(Strings[i], U, 1) + U + Piece(Strings[i], U, 3);
            end;
          if Verified then
            Mult['7']  := 'GMRAVER^'  + BOOLCHAR[Verified];
          if Verifier > 0 then
            Mult['8']  := 'GMRAVERF^'  + IntToStr(Verifier);
          if VerifiedDateTime > 0 then
            Mult['9']  := 'GMRAVERD^'  + FloatToStr(VerifiedDateTime);
          if EnteredInError then
            Mult['10']  := 'GMRAERR^'  + BOOLCHAR[EnteredInError];
          if SignOff then
            Mult['11']  := 'GMRASGN^'  + BOOLCHAR[SignOff];
          if ChartMarked then
            Mult['12']  := 'GMRACHT^'  + BOOLCHAR[ChartMarked];
          if IDBandMarked then
            Mult['13']  := 'GMRAIDBN^'  + BOOLCHAR[IDBandMarked];
          if Length(Observed_Historical) > 0 then
            Mult['14']  := 'GMRAOBHX^'   + Observed_Historical;
          if ReactionDate > 0 then
            Mult['15']  := 'GMRARDT^'  + FloatToStr(ReactionDate);
          if Length(Severity) > 0 then
            Mult['16']  := 'GMRASEVR^'  + Severity;
          with NewComments do if Count > 0 then
            begin
              Mult['17'] := 'GMRACMTS^' + IntToStr(Count);
              for i := 0 to Count - 1 do
                Mult['17,' + IntToStr(i+1)] := Strings[i];
            end;
        end;
      CallBroker;
      Result := '0';
      //Result := Results[0];
    end;
end;

end.
