unit fMedCopy;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  fAutoSz, StdCtrls, ORDtTm, ORCtrls, ORFn, rOrders;

type
  TfrmMedCopy = class(TfrmAutoSz)
    cmdOK: TButton;
    cmdCancel: TButton;
    GroupBox1: TGroupBox;
    radAdmit: TRadioButton;
    radTransfer: TRadioButton;
    radDischarge: TRadioButton;
    chkRelease: TCheckBox;
    lblSpecialty: TLabel;
    cboSpecialty: TORComboBox;
    calEffective: TORDateBox;
    lblEffective: TLabel;
    procedure chkReleaseClick(Sender: TObject);
    procedure radAdmitClick(Sender: TObject);
    procedure radTransferClick(Sender: TObject);
    procedure radDischargeClick(Sender: TObject);
    procedure cmdOKClick(Sender: TObject);
    procedure cmdCancelClick(Sender: TObject);
    procedure FormCreate(Sender: TObject);
  private
    FDelayEvent: TOrderDelayEvent;
    FOKPressed: Boolean;
  public
    { Public declarations }
  end;

function SetDelayEventForMed(var ADelayEvent: TOrderDelayEvent; LimitEvent: Char): Boolean;

implementation

{$R *.DFM}
uses  fODBase;

const
  TX_SEL_DATE = 'An effective date (approximate) must be selected for discharge orders.';
  TC_SEL_DATE = 'Missing Effective Date';
  TX_SEL_SPEC = 'A treating specialty must be selected for admission & transfer orders.';
  TC_SEL_SPEC = 'Missing Treating Specialty';

function SetDelayEventForMed(var ADelayEvent: TOrderDelayEvent; LimitEvent: Char): Boolean;
var
  frmMedCopy: TfrmMedCopy;
begin
  if LimitEvent = 'C' then  // writing orders to be released immediately
  begin
    Result := True;
    ADelayEvent.EventType := 'C';
    ADelayEvent.Specialty := 0;
    ADelayEvent.Effective := 0;
    Exit;
  end;
  Result := False;
  frmMedCopy := TfrmMedCopy.Create(Application);
  try
    ResizeFormToFont(TForm(frmMedCopy));
    case LimitEvent of
    'A': begin
           frmMedCopy.chkRelease.Checked   := False;
           frmMedCopy.chkRelease.Enabled   := False;
           frmMedCopy.radAdmit.Checked     := True;
           frmMedCopy.radTransfer.Enabled  := False;
           frmMedCopy.radDischarge.Enabled := False;
           frmMedCopy.radAdmitClick(frmMedCopy);
         end;
    'D': begin
           frmMedCopy.chkRelease.Checked   := True;
           frmMedCopy.chkRelease.Enabled   := True;
           frmMedCopy.radDischarge.Checked := False;
           frmMedCopy.radAdmit.Enabled     := False;
           frmMedCopy.radTransfer.Enabled  := False;
           frmMedCopy.radDischargeClick(frmMedCopy);
         end;
    end;
    ListSpecialties(frmMedCopy.cboSpecialty.Items);
    frmMedCopy.ShowModal;
    if frmMedCopy.FOKPressed then
    begin
      if (frmMedCopy.chkRelease.Checked) and (LimitEvent = 'D') then
      begin
        XfInToOutNow := True;
        frmMedCopy.FDelayEvent.EventType := 'D';
        frmMedCopy.FDelayEvent.Specialty := 0;
        frmMedCopy.FDelayEvent.Effective := frmMedCopy.calEffective.FMDateTime;
      end
      else XfInToOutNow := False;
      Result := True;
      ADelayEvent.EventType := frmMedCopy.FDelayEvent.EventType;
      ADelayEvent.Specialty := frmMedCopy.FDelayEvent.Specialty;
      ADelayEvent.Effective := frmMedCopy.FDelayEvent.Effective;
    end;
  finally
    frmMedCopy.Release;
  end;
end;

procedure TfrmMedCopy.FormCreate(Sender: TObject);
begin
  inherited;
  FOKPressed := False;
end;

procedure TfrmMedCopy.chkReleaseClick(Sender: TObject);
begin
  inherited;
  if chkRelease.Checked then
  begin
    radAdmit.Checked     := False;
    radTransfer.Checked  := False;
    radDischarge.Checked := False;
    lblSpecialty.Visible := False;
    cboSpecialty.Visible := False;
    lblEffective.Visible := False;
    calEffective.Visible := False;
  end;
end;

procedure TfrmMedCopy.radAdmitClick(Sender: TObject);
begin
  inherited;
  chkRelease.Checked   := False;
  lblSpecialty.Visible := True;
  cboSpecialty.Visible := True;
  lblEffective.Visible := False;
  calEffective.Visible := False;
end;

procedure TfrmMedCopy.radTransferClick(Sender: TObject);
begin
  inherited;
  chkRelease.Checked   := False;
  lblSpecialty.Visible := True;
  cboSpecialty.Visible := True;
  lblEffective.Visible := False;
  calEffective.Visible := False;
end;

procedure TfrmMedCopy.radDischargeClick(Sender: TObject);
begin
  inherited;
  lblSpecialty.Visible := False;
  cboSpecialty.Visible := False;
  if radDischarge.Checked then
  begin
    chkRelease.Checked := False;
    lblEffective.Visible := True;
    calEffective.Visible := True;
  end;
end;

procedure TfrmMedCopy.cmdOKClick(Sender: TObject);
begin
  inherited;
  if chkRelease.Checked then
  begin
    FDelayEvent.EventType := 'C';
    FDelayEvent.Specialty := 0;
    FDelayEvent.Effective := 0;
  end;
  if radAdmit.Checked then
  begin
    if ShowMsgOn(not (cboSpecialty.ItemIEN > 0), TX_SEL_SPEC, TC_SEL_SPEC) then Exit;
    FDelayEvent.EventType := 'A';
    FDelayEvent.Specialty := cboSpecialty.ItemIEN;
    FDelayEvent.Effective := 0;
  end;
  if radTransfer.Checked then
  begin
    if ShowMsgOn(not (cboSpecialty.ItemIEN > 0), TX_SEL_SPEC, TC_SEL_SPEC) then Exit;
    FDelayEvent.EventType := 'T';
    FDelayEvent.Specialty := cboSpecialty.ItemIEN;
    FDelayEvent.Effective := 0;
  end;
  if radDischarge.Checked then
  begin
    if ShowMsgOn(not (calEffective.FMDateTime > 0), TX_SEL_DATE, TC_SEL_DATE) then Exit;
    FDelayEvent.EventType := 'D';
    FDelayEvent.Specialty := 0;
    FDelayEvent.Effective := calEffective.FMDateTime;
  end;
  FOKPressed := True;
  Close;
end;

procedure TfrmMedCopy.cmdCancelClick(Sender: TObject);
begin
  inherited;
  Close;
end;

end.
