unit fReview;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, checklst, uConst, ExtCtrls, uCore, mCoPayDesc, XUDIGSIGSC_TLB;

type
  TfrmReview = class(TForm)
    cmdOK: TButton;
    cmdCancel: TButton;
    lstReview: TCheckListBox;
    pnlOrderAction: TPanel;
    radSignChart: TRadioButton;
    pnlSignature: TPanel;
    txtESCode: TEdit;
    lblESCode: TLabel;
    Label1: TLabel;
    radHoldSign: TRadioButton;
    grpRelease: TGroupBox;
    radVerbal: TRadioButton;
    radPhone: TRadioButton;
    radPolicy: TRadioButton;
    radRelease: TRadioButton;
    lblSig: TLabel;
    fraCoPay: TfraCoPayDesc;
    procedure FormCreate(Sender: TObject);
    procedure lstReviewDrawItem(Control: TWinControl; Index: Integer;
      Rect: TRect; State: TOwnerDrawState);
    procedure cmdOKClick(Sender: TObject);
    procedure cmdCancelClick(Sender: TObject);
    procedure lstReviewClickCheck(Sender: TObject);
    procedure radReleaseClick(Sender: TObject);
    procedure txtESCodeChange(Sender: TObject);
    procedure lstReviewMeasureItem(Control: TWinControl; Index: Integer;
      var Height: Integer);
    procedure FormDestroy(Sender: TObject);
    procedure lstReviewMouseMove(Sender: TObject; Shift: TShiftState; X,
      Y: Integer);
  private
    { Private declarations }
    FOKPressed: Boolean;
    FShowPanel: Integer;
    FSilent: Boolean;
    FCouldSign: Boolean;
    FLastHintItem: integer;
    FOldHintPause: integer;
    FOldHintHidePause: integer;
    FShrunk: boolean;
    procedure AddHeader(s: string);
    function AddItem(ChangeItem: TChangeItem): integer;
    procedure BuildList(FullList: boolean);
    procedure BuildFullList;
    procedure BuildSignList;
    procedure CleanupChangesList(Sender: TObject; ChangeItem: TChangeItem);  {**RV**}
    function ItemsAreChecked: Boolean;
    function SignRequiredForAny(FullList: boolean): Boolean;
  public                       
    { Public declarations }
  end;

function ReviewChanges(TimedOut: Boolean): Boolean;

var
  crypto: IXuDigSigS;

implementation

{$R *.DFM}

uses ORFn, rCore, fNotes, fConsults, fOrders, rOrders, Hash, fDCSumm, fOCSession, uOrders,
     fSignItem, fOrdersPrint, fLkUpLocation, fFrame, uSignItems, fSurgery;

const
  SP_NONE  = 0;
  SP_CLERK = 1;
  SP_NURSE = 2;
  SP_SIGN  = 3;
  TXT_ENCNT     = 'Outpatient Encounter';
  TXT_NOVISIT   = 'Visit Type: < None Selected >';
  TXT_NODIAG    = 'Diagnosis: < None Selected >';
  TXT_NOPROC    = 'Procedures: none';
  TXT_DOCS      = 'Documents';
  TXT_ORDERS    = 'Orders';
  TXT_BLANK     = ' ';
  TX_INVAL_MSG  = 'Not a valid electronic signature code.  Enter a valid code or press Cancel.';
  TX_INVAL_CAP  = 'Unrecognized Signature Code';
  TX_ES_REQ     = 'Enter your electronic signature to release these orders.';
  TC_ES_REQ     = 'Electronic Signature';
  TX_NO_REL     = CRLF + CRLF + '- cannot be released to the service(s).' + CRLF + CRLF + 'Reason: ';
  TC_NO_REL     = 'Unable to Release Orders';

function ReviewChanges(TimedOut: Boolean): Boolean;
{ display changes made to chart for this encounter, allow changes to be saved, signed, etc. }
var
  frmReview: TfrmReview;
  i: integer;
begin
  Result := True;
  if Changes.Count = 0 then Exit;
  frmReview := TfrmReview.Create(Application);
  try
    Changes.OnRemove := frmReview.CleanupChangesList;     {**RV**}
//    ResizeFormToFont(TForm(frmReview));
    frmReview.lstReview.ItemHeight := MainFontHeight;
    // if user times out, process as if unchecked and saved - sends alerts - {v14}
    if TimedOut and (Changes.Count > 0) then
    begin
      frmReview.FSilent := True;
      frmReview.BuildFullList;
      with frmReview.lstReview do for i := 0 to Items.Count - 1 do
        Checked[i] := False;
      frmReview.cmdOKClick(frmReview);
      Result := True;
    end
    // if user not timed out, execute as before
    else
    begin
      if ((User.OrderRole = OR_NURSE) or (User.OrderRole = OR_CLERK)) and Changes.CanSign then
      begin
        frmReview.FCouldSign := True;
        frmReview.BuildSignList;  // ok will remove from changes, exit leaves altogether
        frmReview.ShowModal;
        Result := frmReview.FOKPressed;
      end;
      if Result and (Changes.Count > 0) then
      begin
        frmReview.FCouldSign := Changes.CanSign;
        frmReview.BuildFullList;
        frmReview.ShowModal;
        Result := frmReview.FOKPressed;
      end;
    end;
    // if Result then Changes.Clear;  possibly clear changes here ??
  finally
    Changes.OnRemove := nil;     {**RV**}
    frmReview.Release;
  end;
end;

procedure TfrmReview.FormCreate(Sender: TObject);
begin
  FOKPressed := False;
  FSilent := False;
  FLastHintItem := -1;
  FOldHintPause := Application.HintPause;
  Application.HintPause := 250;
  FOldHintHidePause := Application.HintHidePause;
  Application.HintHidePause := 30000;
end;

procedure TfrmReview.AddHeader(s: string);
{ add header to review list, object is left nil }
begin
  lstReview.Items.AddObject(s, nil);
end;

function TfrmReview.AddItem(ChangeItem: TChangeItem): integer;
{ add a single review item to the list with its associated TChangeItem object }
begin
  Result := lstReview.Items.AddObject(ChangeItem.Text, ChangeItem);
  case ChangeItem.SignState of
  CH_SIGN_YES: lstReview.Checked[Result] := True;
  CH_SIGN_NO:  lstReview.Checked[Result] := False;
  CH_SIGN_NA:  lstReview.State[Result] := cbGrayed;
  end;
end;

procedure TfrmReview.BuildList(FullList: boolean);
var
  GrpIndex, ChgIndex, lbIdx: Integer;
  ChangeItem: TChangeItem;
  
begin
  lstReview.Clear;  // ok to clear without freeing objects since they're part of Changes
  if(FullList) then
  begin
    SigItems.ResetOrders;
    with Changes do if PCE.Count > 0 then
    begin
      for GrpIndex := 0 to PCEGrp.Count - 1 do
      begin
        AddHeader('Outpatient Encounter ' + PCEGrp[GrpIndex]);
        for ChgIndex := 0 to PCE.Count - 1 do
        begin
          ChangeItem := PCE[ChgIndex];
          if ChangeItem.GroupName = PCEGrp[GrpIndex] then AddItem(ChangeItem);
        end;
        AddHeader('   ');
      end;
    end; {if PCE}
  end;
  with Changes do if Documents.Count > 0 then
  begin
    AddHeader('Documents');
    for ChgIndex := 0 to Documents.Count - 1 do
    begin
      ChangeItem := Documents[ChgIndex];
      if(FullList or (ChangeItem.SignState <> CH_SIGN_NA)) then
         AddItem(ChangeItem);
    end;
    if(FullList) then
       AddHeader('   ');
  end; {if Documents}
  if(FullList) then
  begin
    with Changes do if Orders.Count > 0 then
    begin
      for GrpIndex := 0 to OrderGrp.Count - 1 do
      begin
        AddHeader('Orders - ' + OrderGrp[GrpIndex]);
        for ChgIndex := 0 to Orders.Count - 1 do
        begin
          ChangeItem := Orders[ChgIndex];
          if ChangeItem.GroupName = OrderGrp[GrpIndex] then
          begin
            lbIdx := AddItem(ChangeItem);
            SigItems.Add(CH_ORD, ChangeItem.ID, lbIdx);
          end;
        end;
        AddHeader('   ');
      end;
    end; {if Orders}
    // determine the appropriate panel to display
    case User.OrderRole of
        OR_CLERK: FShowPanel := SP_CLERK;
        OR_NURSE: FShowPanel := SP_NURSE;
    OR_PHYSICIAN: FShowPanel := SP_SIGN;
      OR_STUDENT: if Changes.CanSign then FShowPanel := SP_SIGN else FShowPanel := SP_NONE;
    else          FShowPanel := SP_NONE;
    end; {case User}
  end
  else
    FShowPanel := SP_SIGN;
    
  case FShowPanel of
  SP_CLERK: begin
              pnlSignature.Visible := False;
              //pnlOrderAction.Visible := True;
              pnlOrderAction.Visible := SignRequiredForAny(FullList);
            end;
  SP_NURSE: begin
              pnlSignature.Visible := False;
              radRelease.Visible := True;
              grpRelease.Visible := True;
              pnlOrderAction.Visible := SignRequiredForAny(FullList);
            end;       
   SP_SIGN: begin
              pnlOrderAction.Visible := False;
              pnlSignature.Visible := ItemsAreChecked;
            end;
  else      begin                                // SP_NONE
              pnlOrderAction.Visible := False;
              pnlSignature.Visible := False;
            end;
  end; {case FShowPanel}
  txtESCodeChange(Self);
  if pnlOrderAction.Visible then
  begin
    lblSig.Caption := 'Documents / Orders';
    if FShowPanel = SP_NURSE then
    begin
      if GetUserParam('OR SIGNATURE DEFAULT ACTION') = 'OC'
        then radHoldSign.Checked  := True
        else radRelease.Checked   := True;
    end;
    if (radHoldSign.Checked) and (GetUserParam('OR SIGNED ON CHART') = '1')
      then radSignChart.Checked := True;
    if radRelease.Checked then radReleaseClick(Self);
  end {if pnlOrderAction}
  else
    lblSig.Caption := 'Signature will be Applied to Checked Items';
  if (FullList and SigItems.UpdateListBox(lstReview)) then
  begin
    fraCoPay.Visible := TRUE;
    if(FShrunk) then
    begin
      FShrunk := FALSE;
      height := height + fraCoPay.Height + 9;
    end;                                      
  end
  else
  begin
    fraCoPay.Visible := FALSE;
    if(not FShrunk) then
    begin
      FShrunk := TRUE;
      height := height - fraCoPay.Height - 9;
    end;
  end;
end; {BuildFullList}


procedure TfrmReview.BuildFullList;
begin
   BuildList(TRUE);
end;

procedure TfrmReview.BuildSignList;
begin
   BuildList(FALSE);
end;

function TfrmReview.ItemsAreChecked: Boolean;
{ return true if any items in the Review List are checked for applying signature }
var
  i: Integer;
begin
  Result := False;
  with lstReview do for i := 0 to Items.Count - 1 do if Checked[i] then
  begin
    Result := True;
    break;
  end;
end;

function TfrmReview.SignRequiredForAny(FullList: boolean): Boolean;
var
  i: Integer;
  tmpOrders: TStringList;
  ChangeItem: TChangeItem;
begin
  Result := True;
  if Changes.Documents.Count > 0 then Exit;
  if(FullList) then
  begin
  //  if Changes.PCE.Count > 0 then Exit;   *** JM - you don't sign encounter informaiton ***
    tmpOrders := TStringList.Create;
    try
      for i := 0 to Pred(Changes.Orders.Count) do
      begin
        ChangeItem := Changes.Orders[i];
        tmpOrders.Add(ChangeItem.ID);
      end;
      Result := AnyOrdersRequireSignature(tmpOrders);
    finally
      tmpOrders.Free;
    end;
  end
  else
    Result := FALSE;
end;

procedure TfrmReview.lstReviewClickCheck(Sender: TObject);
{ prevent grayed checkboxes from being changed to anything else }
var
  ChangeItem: TChangeItem;
begin
  with lstReview do
  begin
    ChangeItem := TChangeItem(Items.Objects[ItemIndex]);
    if (ChangeItem <> nil) then
    begin
      if (ChangeItem.SignState = CH_SIGN_NA)
        then State[ItemIndex] := cbGrayed
      else
        SigItems.EnableSettings(ItemIndex, checked[ItemIndex]);
    end;
  end;
  pnlSignature.Visible := ItemsAreChecked;
end;

procedure TfrmReview.lstReviewDrawItem(Control: TWinControl; Index: Integer; Rect: TRect;
  State: TOwnerDrawState);
{ outdent the header items (thus hiding the checkbox) }
begin
  with lstReview do
  begin
    if Items.Objects[Index] = nil then Rect.Left := 0;
    //if ReviewItem.HighlightText then Canvas.Font.Color := clBlue;
    Canvas.FillRect(Rect);
    if Index < Items.Count then Canvas.TextOut(Rect.Left + 2, Rect.Top, FilteredString(Items[Index]));
  end;
end;

procedure TfrmReview.radReleaseClick(Sender: TObject);
begin
  if not grpRelease.Visible then Exit;
  if radRelease.Checked then
  begin
    radVerbal.Enabled := True;
    radPhone.Enabled  := True;
    radPolicy.Enabled := True;
    if Encounter.Provider = User.DUZ
      then radPolicy.Checked := True
      else radVerbal.Checked := True;
  end else
  begin
    radVerbal.Enabled := False;
    radPhone.Enabled  := False;
    radPolicy.Enabled := False;
    radVerbal.Checked := False;
    radPhone.Checked  := False;
    radPolicy.Checked := False;
  end;
end;

procedure TfrmReview.txtESCodeChange(Sender: TObject);
begin
  if(not pnlSignature.Visible) then
    cmdOK.Caption := 'OK'
  else
  begin
    if Length(txtESCode.Text) > 0 then cmdOK.Caption := 'Sign' else
    begin
      if FCouldSign then cmdOK.Caption := 'Don''t Sign' else cmdOK.Caption := 'OK';
    end;
  end;
end;

procedure TfrmReview.cmdOKClick(Sender: TObject);
{ validate the electronic signature & call SaveSignItem for the encounter }
const
  TX_NOSIGN  = 'Save items without signing?';
  TC_NOSIGN  = 'No Signature Entered';
  TX_SAVERR1 = 'The error, ';
  TX_SAVERR2 = ', occurred while trying to save:' + CRLF + CRLF;
  TC_SAVERR  = 'Error Saving Order';
var
  i, AType: Integer;
  SigSts, RelSts, Nature: Char;
  ESCode, AnID, AnErrMsg: string;
  ChangeItem: TChangeItem;
  OrderList{, PrintList, NewOrderList}: TStringList;
  SaveCoPay: boolean;
  DigSigErr, DigStoreErr: Boolean;
  SigData, SigUser, SigDrugSch, SigDEA: string;
  cSignature, cHashData, cCrlUrl: string;
  cProvDUZ: Int64;

  function OrdersSignedOrReleased: Boolean;
  var
    i: Integer;
  begin
    Result := False;
    for i := 0 to Pred(OrderList.Count) do
    begin
      if Pos('R', Piece(OrderList[i], U, 2)) > 0 then Result := True;
      if Pos('S', Piece(OrderList[i], U, 2)) > 0 then Result := True;
      if Result then Break;
    end;
  end;

  function OrdersToBeSignedOrReleased: Boolean;
  var
    i: Integer;
    s: string;

  begin
    Result := False;
    for i := 0 to Pred(OrderList.Count) do
    begin
      s := Piece(OrderList[i], U, 2);
      if ((s <> '') and (s[1] in [SS_ONCHART, SS_ESIGNED, SS_NOTREQD])) or
         (Piece(OrderList[i], U, 3) = RS_RELEASE) then
      begin
        Result := TRUE;
        break;
      end;
    end;
  end;


begin
  ESCode := '';
  SaveCoPay := FALSE;

  if pnlSignature.Visible then
  begin
    ESCode := txtESCode.Text;
    //if ItemsAreChecked and (ESCode = '') then
    //  if InfoBox(TX_NOSIGN, TC_NOSIGN, MB_YESNO) = ID_NO then Exit;
    if ItemsAreChecked and (Length(ESCode) > 0) and (not ValidESCode(ESCode)) then
    begin
      InfoBox(TX_INVAL_MSG, TX_INVAL_CAP, MB_OK);
      txtESCode.SetFocus;
      txtESCode.SelectAll;
      Exit;
    end;
    if Length(ESCode) > 0 then ESCode := Encrypt(ESCode);
  end; {if pnlSignature}

  if not frmFrame.Closing then
  begin
    { save/sign orders }
    OrderList := TStringList.Create;
    {PrintList := TStringList.Create;
    NewOrderList := TStringList.Create;}
    DigSigErr := True;
    DigStoreErr := False;
    if GetPKIUse then
      begin
        try  //PKI object creation
          crypto := CoXuDigSigS.Create;
          crypto.GetCSP;
          StatusText(crypto.Reason);
          DigSigErr := False;
        except
        on  E: Exception do
          begin
            DigSigErr := True;
            ShowMessage('Error while trying to Digitally Sign: '+ piece(E.Message,'^',2));
          end;
        end;
      end;
    try
      Nature := NO_PROVIDER;
      case User.OrderRole of
      OR_NOKEY, OR_CLERK, OR_NURSE, OR_STUDENT:
        begin
          SigSts := SS_UNSIGNED;                   // default to med student values
          RelSts := RS_HOLD;
          Nature := NO_WRITTEN;
          if User.OrderRole in [OR_CLERK, OR_NURSE] then
          begin
            if radSignChart.Checked then SigSts := SS_ONCHART else SigSts := SS_UNSIGNED;
            if radRelease.Checked or radSignChart.Checked
              then RelSts := RS_RELEASE
              else RelSts := RS_HOLD;
            if radSignChart.Checked or radHoldSign.Checked then Nature := NO_WRITTEN
            else if radVerbal.Checked then Nature := NO_VERBAL
            else if radPhone.Checked  then Nature := NO_PHONE
            else if radPolicy.Checked then Nature := NO_POLICY
            else Nature := NO_WRITTEN;
            if not pnlOrderAction.Visible then     // if no orders require a signature
            begin
              RelSts := RS_RELEASE;
              Nature := NO_PROVIDER;
              SigSts := SS_NOTREQD;
            end;
            // the following was added due to patch OR*3.0*86
            if RelSts = RS_RELEASE then
            begin
              StatusText('Validating Release...');
              AnErrMsg := '';
              for i := 0 to lstReview.Items.Count - 1 do
              begin
                ChangeItem := TChangeItem(lstReview.Items.Objects[i]);
                if (ChangeItem <> nil) and (ChangeItem.ItemType = CH_ORD) then
                begin
                  ValidateOrderActionNature(ChangeItem.ID, OA_RELEASE, Nature, AnErrMsg);
                  if Length(AnErrMsg) > 0 then
                  begin
                    InfoBox(ChangeItem.Text + TX_NO_REL + AnErrMsg, TC_NO_REL, MB_OK);
                    Break;
                  end; {if Length(AnErrMsg)}
                end; {if ChangeItem=CH_ORD}
              end; {for}
              StatusText('');
              if Length(AnErrMsg) > 0 then Exit;
            end; {if RelSts}
            // the following supports the change to allow nurses to sign policy orders
            if (RelSts = RS_RELEASE) and pnlOrderAction.Visible then
            begin
              SignatureForItem(Font.Size, TX_ES_REQ, TC_ES_REQ, ESCode);
              if ESCode = '' then Exit;
              if Nature = NO_POLICY then SigSts := SS_ESIGNED;
            end;
          end; {if..ORCLERK, OR_NURSE}
          with lstReview do for i := 0 to Items.Count - 1 do
          begin
            ChangeItem := TChangeItem(Items.Objects[i]);
            if (ChangeItem <> nil) and (ChangeItem.ItemType = CH_ORD) then
              OrderList.Add(ChangeItem.ID + U + SigSts + U + RelSts + U + Nature);
          end; {with lstReview}
        end; {OR_NOKEY, OR_CLERK, OR_NURSE, OR_STUDENT}
      OR_PHYSICIAN:
        begin
          Nature := NO_PROVIDER;
          with lstReview do for i := 0 to Items.Count - 1 do
          begin
            ChangeItem := TChangeItem(Items.Objects[i]);
            if (ChangeItem <> nil) and (ChangeItem.ItemType = CH_ORD) then
              begin
                case State[i] of
                cbChecked:   if Length(ESCode) > 0 then
                             begin
                               SigSts := SS_ESIGNED;
                               RelSts := RS_RELEASE;
                             end else
                             begin
                               SigSts := SS_UNSIGNED;
                               RelSts := RS_HOLD;
                             end;
                cbGrayed:    if OrderRequiresSignature(ChangeItem.ID) then
                             begin
                               SigSts := SS_UNSIGNED;
                               RelSts := RS_HOLD;
                             end else
                             begin
                               SigSts := SS_NOTREQD;
                               RelSts := RS_RELEASE;
                             end;
                else         begin                   // (cbUnchecked)
                               SigSts := SS_UNSIGNED;
                               RelSts := RS_HOLD;
                             end;
                end; {case State}
                if (DigSigErr = false) and (OrderRequiresDigitalSignature(ChangeItem.ID)) then
                  begin
                    StatusText('Retrieving DIGITAL SIGNATURE');
                    SigData := GetExternalText(ChangeItem.ID);
                    SigUser := piece(SigData,'^',12);
                    SigDrugSch := piece(SigData,'^',10);
                    SigDEA := piece(SigData,'^',14);
                    cProvDUZ := StrToInt64(piece(SigData,'^',13));
                    if SigDrugSch = '1' then SigDrugSch := '2'; //Change after new API from RX
                    if (length(SigData)<1) or (length(SigUser)<1) or (length(SigDEA)<1) then
                      begin
                        OrderList.Add(ChangeItem.ID + U + SS_ESIGNED + U + RS_RELEASE + U + NO_PROVIDER);
                        continue;
                      end;
                    try
                      crypto.Reset;
                      crypto.DEAsig := true;
                      crypto.UsrName := SigUser;
                      crypto.DrugSch := SigDrugSch;
                      crypto.UsrNumber := SigDEA;
                      crypto.DataBuffer := SigData;
                      if crypto.Signdata = true then
                        begin
                          cSignature := crypto.Signature;
                          cHashData := crypto.HashValue;
                          cCrlUrl := crypto.CrlUrl;
                        end
                      else
                        begin
                          ShowMessage('Digital Signature failed with reason: '+ piece(Crypto.Reason, '^', 2));
                          DigStoreErr := true;
                        end;
                    except
                      on  E: Exception do
                        begin
                          ShowMessage('Crypto raised an error: '+ E.Message);
                          DigStoreErr := true;
                        end;
                    end;  //except
                    if DigStoreErr = true then
                      OrderList.Add(ChangeItem.ID + U + SS_ESIGNED + U + RS_RELEASE + U + Nature)
                    else
                      begin
                        StoreDigitalSig(ChangeItem.ID, cHashData, cProvDUZ, cSignature, cCrlUrl);
                        OrderList.Add(ChangeItem.ID + U + SS_DIGSIG + U + RS_RELEASE + U + Nature);
                      end;
                  end
                else
                  if (ChangeItem.GroupName = 'Other Unsigned') and (SigSts = SS_UNSIGNED) and (RelSts = RS_HOLD)
                    then //NoOp - don't add unsigned orders from outside session to the list
                  else
                    OrderList.Add(ChangeItem.ID + U + SigSts + U + RelSts + U + Nature);

                {if (ChangeItem.GroupName = 'Other Unsigned') and (SigSts = SS_UNSIGNED) and (RelSts = RS_HOLD)
                  then //NoOp - don't add unsigned orders from outside session to the list
                  else OrderList.Add(ChangeItem.ID + U + SigSts + U + RelSts + U + Nature);}

            end; {if ItemType}
          end; {with lstReview}
        end; {OR_PHYSICIAN}
      end; {case User.OrderRole}

      if OrdersToBeSignedOrReleased then
      begin
        if (not SigItems.OK2SaveSettings) then
        begin
          InfoBox(TX_CoPay_Error, TC_CoPay_Error, MB_OK);
          Exit;
        end
        else
          SaveCoPay := TRUE;
      end;

      {release & print orders}
      // test for LockedForOrdering is to make sure patient is locked if pulling in all unsigned
      if (User.OrderRole in [OR_NOKEY..OR_STUDENT]) and (OrderList.Count > 0) and LockedForOrdering then
      begin
        ExecuteSessionOrderChecks(OrderList);  // any cancelled orders will be removed from OrderList
        StatusText('Sending Orders to Service(s)...');
        if OrderList.Count > 0 then
        begin
          if SaveCoPay then
            SigItems.SaveSettings; // Save CoPay FIRST
          SendOrders(OrderList, ESCode);   {*KCM*}
        end;
        with OrderList do for i := 0 to Count - 1 do
        begin
          if Pos('E', Piece(OrderList[i], U, 2)) > 0 then
          begin
            ChangeItem := Changes.Locate(CH_ORD, Piece(OrderList[i], U, 1));
            if not FSilent then InfoBox(TX_SAVERR1 + Piece(OrderList[i], U, 4) + TX_SAVERR2 + ChangeItem.Text,
                    TC_SAVERR, MB_OK);
          end;
          if Pos('R', Piece(OrderList[i], U, 2)) > 0 then
            NotifyOtherApps(NAE_ORDER, 'RL' + U + Piece(OrderList[i], U, 1));
        end;
        if OrdersSignedOrReleased and (not FSilent)
          then PrintOrdersOnSignRelease(OrderList, Nature);
        StatusText('');
        UpdateUnsignedOrderAlerts(Patient.DFN);
        with Notifications do
          if Active and (FollowUp = NF_ORDER_REQUIRES_ELEC_SIGNATURE) then
            UnsignedOrderAlertFollowup(Piece(RecordID, U, 2));
        UpdateExpiringMedAlerts(Patient.DFN);
        UpdateUnverifiedMedAlerts(Patient.DFN);
        UpdateUnverifiedOrderAlerts(Patient.DFN);
        SendMessage(Application.MainForm.Handle, UM_NEWORDER, ORDER_SIGN, 0);
      end; {if User.OrderRole}

    finally
      OrderList.Free;
    end;
  end;

  { save/sign documents }
  //with lstReview do for i := 0 to Items.Count - 1 do
  with lstReview do for i := 0 to Items.Count - 1 do
  begin
    ChangeItem := TChangeItem(Items.Objects[i]);
    if ChangeItem <> nil then with ChangeItem do
      case ItemType of
      CH_DOC: if Checked[i]
                then frmNotes.SaveSignItem(ChangeItem.ID, ESCode)
                else frmNotes.SaveSignItem(ChangeItem.ID, '');
      CH_CON: if Checked[i]
                then frmConsults.SaveSignItem(ChangeItem.ID, ESCode)
                else frmConsults.SaveSignItem(ChangeItem.ID, '');
      CH_SUM: if Checked[i]
                then frmDCSumm.SaveSignItem(ChangeItem.ID, ESCode)
                else frmDCSumm.SaveSignItem(ChangeItem.ID, '');
      CH_SUR: if Assigned(frmSurgery) then
              begin
                if Checked[i]
                  then frmSurgery.SaveSignItem(ChangeItem.ID, ESCode)
                  else frmSurgery.SaveSignItem(ChangeItem.ID, '');
              end;
      end; {case}
  end; {with lstReview}
  if frmFrame.Closing then exit;

  // clear all the items that were on the list (but not all in Changes)
  with lstReview do for i := Items.Count - 1 downto 0  do
  begin
    if (not Assigned(Items.Objects[i])) then continue;   {**RV**}
    ChangeItem := TChangeItem(Items.Objects[i]);
    if ChangeItem <> nil then
    begin
      AnID := ChangeItem.ID;
      AType := ChangeItem.ItemType;
      Changes.Remove(AType, AnID);
    end;
  end;

  UnlockIfAble;
  FOKPressed := True;
  Close;
end;

procedure TfrmReview.CleanupChangesList(Sender: TObject; ChangeItem: TChangeItem);
{Added for v15.3 - called by Changes.Remove, but only if fReview in progress}
var
  i: integer;
begin
  with lstReview do
    begin
      i := Items.IndexOfObject(ChangeItem);
      if i > -1 then
        begin
          TChangeItem(Items.Objects[i]).Free;
          Items.Objects[i] := nil;
        end;
    end;
end;

procedure TfrmReview.cmdCancelClick(Sender: TObject);
{ cancelled - do nothing }
begin
  Close;
end;

procedure TfrmReview.lstReviewMeasureItem(Control: TWinControl;
  Index: Integer; var Height: Integer);
begin
  Height := SigItemHeight;
end;

procedure TfrmReview.FormDestroy(Sender: TObject);
begin
  Application.HintPause := FOldHintPause;
  Application.HintHidePause := FOldHintHidePause;
end;

procedure TfrmReview.lstReviewMouseMove(Sender: TObject;
  Shift: TShiftState; X, Y: Integer);
var
  Itm: integer;

begin
  inherited;
  Itm := lstReview.ItemAtPos(Point(X, Y), TRUE);
  if (Itm >= 0) then
  begin
    if (Itm <> FLastHintItem) then
    begin
      Application.CancelHint;
      lstReview.Hint := TrimRight(lstReview.Items[Itm]);
      FLastHintItem := Itm;
      Application.ActivateHint(Point(X, Y));
    end;
  end
  else
  begin
    lstReview.Hint := '';
    FLastHintItem := -1;
    Application.CancelHint;
  end;
end;

end.
