unit uSpell;

interface

uses
  Windows, Messages, SysUtils, Classes, Controls, Forms, ComObj, StdCtrls, ORSystem;

function SpellCheckAvailable: Boolean;
procedure SpellCheckForControl(AnEditControl: TCustomEdit);
procedure GrammarCheckForControl(AnEditControl: TCustomEdit);

implementation

var
  WindowList: TList;

function SpellCheckTitle: string;
begin
  Result := 'CPRS-Chart Spell Checking #' + IntToStr(Application.Handle);
end;

function GetWindows(Handle: HWND; Info: Pointer): BOOL; stdcall;
begin
  Result := True;
  WindowList.Add(Pointer(Handle));
end;

procedure GetWindowList(List: TList);
begin
  WindowList := List;
  EnumWindows(@GetWindows, 0);
end;

procedure BringWordToFront(OldList, NewList: TList);
var
  i, NameLen: integer;
  WinName: array[0..160] of char;
  NewName: string;

begin
  NewName := SpellCheckTitle;
  NameLen := length(NewName);
  for i := 0 to NewList.Count-1 do
  begin
    if(OldList.IndexOf(NewList[i]) < 0) then
    begin
      GetWindowText(HWND(NewList[i]), WinName, sizeof(WinName) - 1);
      if StrLComp(WinName, pchar(NewName), NameLen)=0 then
      begin
        Application.ProcessMessages;
        SetForegroundWindow(HWND(NewList[i]));
        break;
      end;
    end;
  end;
end;

{ Spell Checking using Visual Basic for Applications script }

function SpellCheckAvailable: Boolean;
const
  WORD_VBA_CLSID = 'CLSID\{000209FF-0000-0000-C000-000000000046}';
begin

// CHANGED FOR PT. SAFETY ISSUE RELEASE 19.16, PATCH OR*3*155 - NEXT 2 LINES:
result := false;
exit;

  Result := True;
  //Result := RegKeyExists(HKEY_CLASSES_ROOT, WORD_VBA_CLSID);
end;

procedure SpellCheckForControl(AnEditControl: TCustomEdit);
var
  MSWord: OleVariant;
  i: Integer;
  OneChar: Char;
  NoLFText, LFText: string;
  OldList, NewList: TList;

begin
  if AnEditControl = nil then Exit;
  OldList := TList.Create;
  try
    GetWindowList(OldList);
    try
      Screen.Cursor := crHourGlass;
      MSWord := CreateOleObject('Word.Application');
    except
      Screen.Cursor := crDefault;
      Application.MessageBox('Spell checking is unavailable.', PChar(Application.Title), MB_ICONWARNING);
      Exit;
    end;
    NewList := TList.Create;
    try
      GetWindowList(NewList);
      try
        MSWord.Application.Caption := SpellCheckTitle;
        MSWord.Documents.Add;                              // FileNew
        MSWord.ActiveDocument.CheckSpelling;               // Load spell checking into memory prior to BringToFront
        MSWord.Selection.TypeText(AnEditControl.Text);     // Insert()
        Screen.Cursor := crDefault;
        BringWordToFront(OldList, NewList);
        MSWord.ActiveDocument.CheckSpelling;               // ToolsSpelling
        MSWord.ActiveDocument.Content.Select;              // EditSelectAll
        NoLFText := MSWord.Selection.Text;                 // Selection
      finally
        Screen.Cursor := crDefault;
        MSWord.Quit(False); //wdDoNotSaveChanges?
      end;
    finally
      NewList.Free;
    end;
  finally
    OldList.Free;
  end;
  Application.BringToFront;
  if Length(NoLFText) > 0 then
    begin
      LFText := '';
      for i := 1 to Length(NoLFText) do
      begin
        OneChar := NoLFText[i];
        LFText := LFText + OneChar;
        if OneChar = #13 then LFText := LFText + #10;
      end;
      AnEditControl.Text := LFText;
      Application.MessageBox('The spelling check is complete.', PChar(Application.Title), MB_ICONINFORMATION);
    end
  else
    Application.MessageBox('The spelling check terminated abnormally.' + #13#10 +
                           'Corrections have NOT been applied.', PChar(Application.Title), MB_ICONINFORMATION);
  AnEditControl.SetFocus;
end;

procedure GrammarCheckForControl(AnEditControl: TCustomEdit);
var
  MSWord: OleVariant;
  i: Integer;
  OneChar: Char;
  NoLFText, LFText: string;
  OldList, NewList: TList;

begin
  if AnEditControl = nil then Exit;
  OldList := TList.Create;
  try
    GetWindowList(OldList);
    try
      Screen.Cursor := crHourGlass;
      MSWord := CreateOleObject('Word.Application');
    except
      Screen.Cursor := crDefault;
      Application.MessageBox('Spell checking is unavailable.', PChar(Application.Title), MB_ICONWARNING);
      Exit;
    end;
    NewList := TList.Create;
    try
      GetWindowList(NewList);
      try
        MSWord.Application.Caption := SpellCheckTitle;
        MSWord.Documents.Add;                              // FileNew
        MSWord.ActiveDocument.CheckGrammar;                // Load grammar checking into memory prior to BringToFront
        MSWord.Selection.TypeText(AnEditControl.Text);     // Insert()
        Screen.Cursor := crDefault;
        BringWordToFront(OldList, NewList);
        MSWord.ActiveDocument.CheckGrammar;                // ToolsGrammar
        MSWord.ActiveDocument.Content.Select;              // EditSelectAll
        NoLFText := MSWord.Selection.Text;                 // Selection
      finally
        Screen.Cursor := crDefault;
        MSWord.Quit(False); //wdDoNotSaveChanges?
      end;
    finally
      NewList.Free;
    end;
  finally
    OldList.Free;
  end;
  Application.BringToFront;
  if Length(NoLFText) > 0 then
    begin
      LFText := '';
      for i := 1 to Length(NoLFText) do
      begin
        OneChar := NoLFText[i];
        LFText := LFText + OneChar;
        if OneChar = #13 then LFText := LFText + #10;
      end;
      AnEditControl.Text := LFText;
      Application.MessageBox('The grammar check is complete.', PChar(Application.Title), MB_ICONINFORMATION);
    end
  else
    Application.MessageBox('The grammar check terminated abnormally.' + #13#10 +
                           'Corrections have NOT been applied.', PChar(Application.Title), MB_ICONINFORMATION);
  AnEditControl.SetFocus;
end;

(*
{ Spell Checking using Word Basic script }

function SpellCheckAvailable: Boolean;
const
  WORD_BASIC_CLSID = 'CLSID\{000209FE-0000-0000-C000-000000000046}';
begin
  // Key = HKEY_CLASSES_ROOT\CLSID\{000209FE-0000-0000-C000-000000000046}
  // check subkey, LocalServer32, to see if file exists?
  Result := RegKeyExists(HKEY_CLASSES_ROOT, WORD_BASIC_CLSID);
end;

procedure SpellCheckForControl(AnEditControl: TCustomEdit);
var
  MSWord: OleVariant;
  i: Integer;
  OneChar: Char;
  NoLFText, LFText: string;
begin
  if AnEditControl = nil then Exit;
  try
    Screen.Cursor := crHourGlass;
    MSWord := CreateOleObject('Word.Basic');
    Screen.Cursor := crDefault;
  except
    Screen.Cursor := crDefault;
    Application.MessageBox('Spell checking is unavailable.', PChar(Application.Title), MB_ICONWARNING);
    Exit;
  end;
  try
    MSWord.FileNew;
    MSWord.Insert(AnEditControl.Text);
    MSWord.ToolsSpelling;
    MSWord.EditSelectAll;
    NoLFText := MSWord.Selection;
  finally
    try
      MSWord.FileExit(2);
    except
      MSWord.DocClose(2);
      MSWord := Unassigned;
    end;
  end;
  Application.BringToFront;
  for i := 1 to Length(NoLFText) do
  begin
    OneChar := NoLFText[i];
    LFText := LFText + OneChar;
    if OneChar = #13 then LFText := LFText + #10;
  end;
  AnEditControl.Text := LFText;
  Application.MessageBox('The spelling check is complete.', PChar(Application.Title), MB_ICONINFORMATION);
  AnEditControl.SetFocus;
end;

procedure GrammarCheckForControl(AnEditControl: TCustomEdit);
var
  MSWord: OleVariant;
  i: Integer;
  OneChar: Char;
  NoLFText, LFText: string;
begin
  if AnEditControl = nil then Exit;
  try
    MSWord := CreateOleObject('Word.Basic');
  except
    Application.MessageBox('Grammar checking is unavailable.', PChar(Application.Title), MB_ICONWARNING);
    Exit;
  end;
  try
    MSWord.FileNew;
    MSWord.Insert(AnEditControl.Text);
    MSWord.ToolsGrammar;
    MSWord.EditSelectAll;
    NoLFText := MSWord.Selection;
  finally
    try
      MSWord.FileExit(2);
    except
      MSWord.DocClose(2);
      MSWord := Unassigned;
    end;
  end;
  Application.BringToFront;
  for i := 1 to Length(NoLFText) do
  begin
    OneChar := NoLFText[i];
    LFText := LFText + OneChar;
    if OneChar = #13 then LFText := LFText + #10;
  end;
  AnEditControl.Text := LFText;
  Application.MessageBox('The grammar check is complete.', PChar(Application.Title), MB_ICONINFORMATION);
  AnEditControl.SetFocus;
end;
*)

end.
